@@ -18,12 +18,15 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: metar.cxx,v 1.13 2006-03-08 18:16:08 mfranz Exp $
+// $Id: metar.cxx,v 1.14.2.4 2007-10-13 14:05:48 durk Exp $
 
 /**
  * @file metar.cxx
  * Interface for encoded Meteorological Aerodrome Reports (METAR).
  */
+#ifdef HAVE_CONFIG_H
+#  include <simgear_config.h>
+#endif
 
 #include <string>
 #include <time.h>
@@ -203,7 +206,7 @@ char *SGMetar::loadData(const char *id, const string& proxy, const string& port,
 	if (!proxy.empty())
 		get += "http://weather.noaa.gov";
 
-	sprintf(buf, "%ld", time);
+	snprintf(buf, (buflen*2), "%ld", time);
 	get += path + " HTTP/1.0\015\012X-Time: " + buf + "\015\012";
 
 	if (!auth.empty())
@@ -744,6 +747,14 @@ bool SGMetar::scanSkyCondition()
 	int i;
 	SGMetarCloud cl;
 
+	if (!strncmp(m, "//////", 6)) {
+		m += 6;
+		if (!scanBoundary(&m))
+			return false;
+		_m = m;
+		return true;
+	}
+
 	if (!strncmp(m, "CLR", i = 3)				// clear
 			|| !strncmp(m, "SKC", i = 3)		// sky clear
 			|| !strncmp(m, "NSC", i = 3)		// no significant clouds
@@ -833,7 +844,7 @@ bool SGMetar::scanTemperature()
 		return false;
 	if (!scanBoundary(&m)) {
 		if (!strncmp(m, "XX", 2))	// not spec compliant!
-			m += 2, sign = 0;
+			m += 2, sign = 0, dew = temp;
 		else {
 			sign = 1;
 			if (*m == 'M')
