@@ -18,8 +18,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_serial.cxx,v 1.6 2006-03-08 18:16:08 mfranz Exp $
+// $Id: sg_serial.cxx,v 1.6.2.1 2007-11-27 19:27:43 durk Exp $
 
+#include <stdlib.h>
+#include <cstring>
 
 #include <simgear/compiler.h>
 
