@@ -14,19 +14,18 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Library General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_memory.h,v 1.1.1.1 2002/09/07 02:58:19 curt Exp $
+// $Id: sg_memory.h,v 1.3 2006-03-08 18:16:08 mfranz Exp $
 
 
 #ifndef _SG_MEMORY_H
 #define _SG_MEMORY_H
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#  include <simgear_config.h>
 #endif
 
 #ifdef HAVE_MEMCPY
