@@ -14,12 +14,11 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Library General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: texcoord.cxx,v 1.4 2005/10/25 13:48:58 ehofman Exp $
+// $Id: texcoord.cxx,v 1.6 2006-03-08 18:16:08 mfranz Exp $
 
 
 /* The following is an explanation of our somewhat conveluted and
@@ -140,6 +139,10 @@ enter this in the official comments in case I forget again. :-)
 
 */
 
+#ifdef HAVE_CONFIG_H
+#  include <simgear_config.h>
+#endif
+
 #include <simgear/compiler.h>
 
 // #include STL_IOSTREAM
