@@ -5,7 +5,7 @@
 //
 // This file is in the Public Domain, and comes with no warranty.
 //
-// $Id: condition.cxx,v 1.5 2006-03-14 10:38:06 mfranz Exp $
+// $Id: condition.cxx,v 1.5.2.1 2007-12-06 17:57:46 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear/compiler.h>
@@ -219,7 +219,7 @@ doComparison (const SGPropertyNode * left, const SGPropertyNode *right)
     break;
   }
   }
-  throw sg_exception("Unrecognized node type");
+  throw sg_exception("condition: unrecognized node type in comparison");
   return 0;
 }
 
@@ -304,7 +304,7 @@ readNotCondition( SGPropertyNode *prop_root, const SGPropertyNode *node )
     if (condition != 0)
       return new SGNotCondition(condition);
   }
-  SG_LOG(SG_COCKPIT, SG_ALERT, "Panel: empty 'not' condition");
+  SG_LOG(SG_COCKPIT, SG_ALERT, "empty 'not' condition");
   return 0;
 }
 
@@ -346,8 +346,10 @@ readComparison( SGPropertyNode *prop_root,
   condition->setLeftProperty(prop_root, node->getStringValue("property[0]"));
   if (node->hasValue("property[1]"))
     condition->setRightProperty(prop_root, node->getStringValue("property[1]"));
-  else
+  else if (node->hasValue("value"))
     condition->setRightValue(node->getChild("value", 0));
+  else
+    throw sg_exception("condition: comparison without property[1] or value");
 
   return condition;
 }
