@@ -4,7 +4,7 @@
 //
 // See props.html for documentation [replace with URL when available].
 //
-// $Id: props.cxx,v 1.23 2006-03-14 15:55:24 mfranz Exp $
+// $Id: props.cxx,v 1.26.2.10 2007-10-13 14:05:48 durk Exp $
 
 #include "props.hxx"
 
@@ -116,7 +116,7 @@ parse_name (const string &path, int &i)
       name = ".";
     }
     if (i < max && path[i] != '/')
-      throw string("Illegal character after " + name);
+      throw string("illegal character after " + name);
   }
 
   else if (isalpha(path[i]) || path[i] == '_') {
@@ -295,7 +295,7 @@ find_node (SGPropertyNode * current,
   else if (components[position].name == "..") {
     SGPropertyNode * parent = current->getParent();
     if (parent == 0)
-      throw string("Attempt to move past root with '..'");
+      throw string("attempt to move past root with '..'");
     else
       return find_node(parent, components, position + 1, create);
   }
@@ -595,11 +595,11 @@ void
 SGPropertyNode::trace_write () const
 {
 #if PROPS_STANDALONE
-  cerr << "TRACE: Write node " << getPath () << ", value\""
+  cerr << "TRACE: Write node " << getPath () << ", value \""
        << make_string() << '"' << endl;
 #else
-  SG_LOG(SG_GENERAL, SG_INFO, "TRACE: Write node " << getPath()
-	 << ", value\"" << make_string() << '"');
+  SG_LOG(SG_GENERAL, SG_ALERT, "TRACE: Write node " << getPath()
+	 << ", value \"" << make_string() << '"');
 #endif
 }
 
@@ -613,7 +613,7 @@ SGPropertyNode::trace_read () const
   cerr << "TRACE: Write node " << getPath () << ", value \""
        << make_string() << '"' << endl;
 #else
-  SG_LOG(SG_GENERAL, SG_INFO, "TRACE: Read node " << getPath()
+  SG_LOG(SG_GENERAL, SG_ALERT, "TRACE: Read node " << getPath()
 	 << ", value \"" << make_string() << '"');
 #endif
 }
@@ -739,7 +739,11 @@ SGPropertyNode::SGPropertyNode (const char * name,
     _attr(READ|WRITE),
     _listeners(0)
 {
-  _name = name;
+  int i = 0;
+  _name = parse_name(name, i);
+  if (i != int(strlen(name)) || name[0] == '.')
+    throw string("plain name expected instead of '") + name + '\'';
+
   _local_val.string_val = 0;
 }
 
@@ -756,7 +760,13 @@ SGPropertyNode::~SGPropertyNode ()
     _removedChildren[i]->_parent = 0;
   delete _path_cache;
   clearValue();
-  delete _listeners;
+
+  if (_listeners) {
+    vector<SGPropertyChangeListener*>::iterator it;
+    for (it = _listeners->begin(); it != _listeners->end(); ++it)
+      (*it)->unregister_property(this);
+    delete _listeners;
+  }
 }
 
 
@@ -904,6 +914,22 @@ SGPropertyNode::getChildren (const char * name) const
 }
 
 
+/**
+ * Remove this node and all children from nodes that link to them
+ * in their path cache.
+ */
+void
+SGPropertyNode::remove_from_path_caches ()
+{
+  for (unsigned int i = 0; i < _children.size(); ++i)
+    _children[i]->remove_from_path_caches();
+
+  for (unsigned int i = 0; i < _linkedNodes.size(); i++)
+    _linkedNodes[i]->erase(this);
+  _linkedNodes.clear();
+}
+
+
 /**
  * Remove child by position.
  */
@@ -921,8 +947,8 @@ SGPropertyNode::removeChild (int pos, bool keep)
   if (keep) {
     _removedChildren.push_back(node);
   }
-  if (_path_cache)
-     _path_cache->erase(node->getName()); // EMH - TODO: Take "index" into account!
+
+  node->remove_from_path_caches();
   node->setAttribute(REMOVED, true);
   node->clearValue();
   fireChildRemoved(node);
@@ -933,7 +959,7 @@ SGPropertyNode::removeChild (int pos, bool keep)
 /**
  * Remove a child node
  */
-SGPropertyNode_ptr 
+SGPropertyNode_ptr
 SGPropertyNode::removeChild (const char * name, int index, bool keep)
 {
   SGPropertyNode_ptr ret;
@@ -961,6 +987,24 @@ SGPropertyNode::removeChildren (const char * name, bool keep)
 }
 
 
+/**
+  * Remove a linked node.
+  */
+bool
+SGPropertyNode::remove_linked_node (hash_table * node)
+{
+  for (unsigned int i = 0; i < _linkedNodes.size(); i++) {
+    if (_linkedNodes[i] == node) {
+      vector<hash_table *>::iterator it = _linkedNodes.begin();
+      it += i;
+      _linkedNodes.erase(it);
+      return true;
+    }
+  }
+  return false;
+}
+
+
 const char *
 SGPropertyNode::getDisplayName (bool simplify) const
 {
@@ -1262,7 +1306,7 @@ SGPropertyNode::setIntValue (int value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    sprintf(buf, "%d", value);
+    snprintf(buf, 128, "%d", value);
     result = set_string(buf);
     break;
   }
@@ -1313,7 +1357,7 @@ SGPropertyNode::setLongValue (long value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    sprintf(buf, "%ld", value);
+    snprintf(buf, 128, "%ld", value);
     result = set_string(buf);
     break;
   }
@@ -1364,7 +1408,13 @@ SGPropertyNode::setFloatValue (float value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    sprintf(buf, "%f", value);
+    if (snprintf(buf, 128, "%f", value) >= 128) {
+#if PROPS_STANDALONE
+         cerr << "caught buffer overflow in " << SG_ORIGIN << endl;
+#else
+         SG_LOG(SG_GENERAL, SG_ALERT, "caught buffer overflow in " << SG_ORIGIN);
+#endif
+    }
     result = set_string(buf);
     break;
   }
@@ -1415,7 +1465,14 @@ SGPropertyNode::setDoubleValue (double value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    sprintf(buf, "%f", value);
+    if (snprintf(buf, 128, "%f", value) >= 128) {
+#if PROPS_STANDALONE
+         cerr << "caught buffer overflow in " << SG_ORIGIN << endl;
+#else
+         SG_LOG(SG_GENERAL, SG_ALERT, "caught buffer overflow in " << SG_ORIGIN);
+#endif
+
+    }
     result = set_string(buf);
     break;
   }
@@ -2187,23 +2244,31 @@ SGPropertyNode::hash_table::bucket::get_entry (const char * key, bool create)
   }
 }
 
-void
-SGPropertyNode::hash_table::bucket::erase (const char * key)
+bool
+SGPropertyNode::hash_table::bucket::erase (SGPropertyNode * node)
 {
-  int i;
-  for (i = 0; i < _length; i++) {
-    if (!strcmp(_entries[i]->get_key(), key))
-       break;
-  }
-
-  if (i < _length) {
-    for (++i; i < _length; i++) {
-      _entries[i-1] = _entries[i];
+  for (int i = 0; i < _length; i++) {
+    if (_entries[i]->get_value() == node) {
+      delete _entries[i];
+      for (++i; i < _length; i++) {
+        _entries[i-1] = _entries[i];
+      }
+      _length--;
+      return true;
     }
-     _length--;
   }
+  return false;
 }
 
+void
+SGPropertyNode::hash_table::bucket::clear (SGPropertyNode::hash_table * owner)
+{
+  for (int i = 0; i < _length; i++) {
+    SGPropertyNode * node = _entries[i]->get_value();
+    if (node)
+      node->remove_linked_node(owner);
+  }
+}
 
 SGPropertyNode::hash_table::hash_table ()
   : _data_length(0),
@@ -2213,8 +2278,12 @@ SGPropertyNode::hash_table::hash_table ()
 
 SGPropertyNode::hash_table::~hash_table ()
 {
-  for (unsigned int i = 0; i < _data_length; i++)
-    delete _data[i];
+  for (unsigned int i = 0; i < _data_length; i++) {
+    if (_data[i]) {
+      _data[i]->clear(this);
+      delete _data[i];
+    }
+  }
   delete [] _data;
 }
 
@@ -2248,17 +2317,17 @@ SGPropertyNode::hash_table::put (const char * key, SGPropertyNode * value)
   }
   entry * e = _data[index]->get_entry(key, true);
   e->set_value(value);
+  value->add_linked_node(this);
 }
 
-void
-SGPropertyNode::hash_table::erase (const char * key)
+bool
+SGPropertyNode::hash_table::erase (SGPropertyNode * node)
 {
-   if (_data_length == 0)
-    return;
-  unsigned int index = hashcode(key) % _data_length;
-  if (_data[index] == 0)
-    return;
-  _data[index]->erase(key);
+  for (unsigned int i = 0; i < _data_length; i++)
+    if (_data[i] && _data[i]->erase(node))
+      return true;
+
+  return false;
 }
 
 unsigned int
@@ -2280,12 +2349,9 @@ SGPropertyNode::hash_table::hashcode (const char * key)
 
 SGPropertyChangeListener::~SGPropertyChangeListener ()
 {
-				// This will come back and remove
-				// the current item each time.  Is
-				// that OK?
-  vector<SGPropertyNode *>::iterator it;
-  for (it = _properties.begin(); it != _properties.end(); it++)
-    (*it)->removeChangeListener(this);
+  for (int i = _properties.size() - 1; i >= 0; i--) {
+    _properties[i]->removeChangeListener(this);
+    }
 }
 
 void
@@ -2319,8 +2385,9 @@ SGPropertyChangeListener::unregister_property (SGPropertyNode * node)
 {
   vector<SGPropertyNode *>::iterator it =
     find(_properties.begin(), _properties.end(), node);
-  if (it != _properties.end())
+  if (it != _properties.end()) {
     _properties.erase(it);
+  }
 }
 
 
