@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: mat.cxx,v 1.24 2006-03-08 18:16:09 mfranz Exp $
+// $Id: mat.cxx,v 1.30.2.1 2007-05-13 11:52:45 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +43,9 @@ SG_USING_STD(map);
 
 #include "mat.hxx"
 
+static map<string, ssgTexture *> _tex_cache;
+static map<string, ssgTexture *>::iterator _tex_cache_iter;
+
 
 ////////////////////////////////////////////////////////////////////////
 // Constructors and destructor.
@@ -120,6 +123,7 @@ SGMaterial::read_properties( const string &fg_root, const SGPropertyNode * props
     string tname = "unknown.rgb";
     SGPath tpath( fg_root );
     tpath.append("Textures");
+    tpath.append("Terrain");
     tpath.append(tname);
     _internal_state st( NULL, tpath.str(), true );
     _status.push_back( st );
@@ -132,6 +136,13 @@ SGMaterial::read_properties( const string &fg_root, const SGPropertyNode * props
   mipmap = props->getBoolValue("mipmap", true);
   light_coverage = props->getDoubleValue("light-coverage", 0.0);
 
+  // surface values for use with ground reactions
+  solid = props->getBoolValue("solid", true);
+  friction_factor = props->getDoubleValue("friction-factor", 1.0);
+  rolling_friction = props->getDoubleValue("rolling-friction", 0.02);
+  bumpiness = props->getDoubleValue("bumpiness", 0.0);
+  load_resistance = props->getDoubleValue("load-resistance", 1e30);
+
   // Taken from default values as used in ac3d
   ambient[0] = props->getDoubleValue("ambient/r", 0.2);
   ambient[1] = props->getDoubleValue("ambient/g", 0.2);
@@ -159,6 +170,14 @@ SGMaterial::read_properties( const string &fg_root, const SGPropertyNode * props
     ((SGPropertyNode *)props)->getChildren("object-group");
   for (unsigned int i = 0; i < object_group_nodes.size(); i++)
     object_groups.push_back(new SGMatModelGroup(object_group_nodes[i]));
+
+  // read glyph table for taxi-/runway-signs
+  vector<SGPropertyNode_ptr> glyph_nodes = props->getChildren("glyph");
+  for (unsigned int i = 0; i < glyph_nodes.size(); i++) {
+    const char *name = glyph_nodes[i]->getStringValue("name");
+    if (name)
+      glyphs[name] = new SGMaterialGlyph(glyph_nodes[i]);
+  }
 }
 
 
@@ -176,8 +195,16 @@ SGMaterial::init ()
     ysize = 0;
     wrapu = true;
     wrapv = true;
+
     mipmap = true;
     light_coverage = 0.0;
+
+    solid = true;
+    friction_factor = 1;
+    rolling_friction = 0.02;
+    bumpiness = 0;
+    load_resistance = 1e30;
+
     shininess = 1.0;
     for (int i = 0; i < 4; i++) {
         ambient[i]  = (i < 3) ? 0.2 : 1.0;
@@ -198,9 +225,8 @@ SGMaterial::load_texture ( int n )
         if ( !_status[i].texture_loaded ) {
             SG_LOG( SG_GENERAL, SG_INFO, "Loading deferred texture "
                                           << _status[i].texture_path );
-            _status[i].state->setTexture(
-                   (char *)_status[i].texture_path.c_str(),
-                   wrapu, wrapv, mipmap );
+            assignTexture(_status[i].state, _status[i].texture_path,
+                                         wrapu, wrapv, mipmap );
             _status[i].texture_loaded = true;
        }
     }
@@ -244,8 +270,7 @@ SGMaterial::build_ssg_state( bool defer_tex_load )
 
         if ( !defer_tex_load ) {
             SG_LOG(SG_INPUT, SG_INFO, "    " << _status[i].texture_path );
-	    state->setTexture( (char *)_status[i].texture_path.c_str(),
-                                wrapu, wrapv );
+	    assignTexture( state, _status[i].texture_path, wrapu, wrapv );
             _status[i].texture_loaded = true;
         } else {
             _status[i].texture_loaded = false;
@@ -276,4 +301,42 @@ void SGMaterial::set_ssg_state( ssgSimpleState *s )
     _status.push_back( _internal_state( s, "", true ) );
 }
 
+void SGMaterial::assignTexture( ssgSimpleState *state, string &fname,
+                 int _wrapu, int _wrapv, int _mipmap )
+{
+   _tex_cache_iter = _tex_cache.find(fname);
+   if (_tex_cache_iter == _tex_cache.end())
+   {
+      state->setTexture((char *)fname.c_str(), _wrapu, _wrapv, _mipmap);
+      _tex_cache[fname] = state->getTexture();
+   }
+   else
+   {
+      state->setTexture(_tex_cache_iter->second);
+      // cout << "Cache hit: " << fname << endl;
+   }
+}
+
+SGMaterialGlyph* SGMaterial::get_glyph (const string& name) const
+{
+  map<string, SGSharedPtr<SGMaterialGlyph> >::const_iterator it;
+  it = glyphs.find(name);
+  if (it == glyphs.end())
+    return 0;
+
+  return it->second;
+}
+
+
+////////////////////////////////////////////////////////////////////////
+// SGMaterialGlyph.
+////////////////////////////////////////////////////////////////////////
+
+SGMaterialGlyph::SGMaterialGlyph(SGPropertyNode *p) :
+    _left(p->getDoubleValue("left", 0.0)),
+    _right(p->getDoubleValue("right", 1.0))
+{
+}
+
+
 // end of mat.cxx
