@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: matlib.cxx,v 1.19 2006-03-08 18:16:09 mfranz Exp $
+// $Id: matlib.cxx,v 1.23 2006-10-19 03:36:22 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,8 @@
 #  include <windows.h>
 #endif
 
+#include <plib/ssg.h>
+
 #include <simgear/compiler.h>
 #include <simgear/constants.h>
 #include <simgear/structure/exception.hxx>
@@ -46,6 +48,7 @@
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sgstream.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/scene/tgdb/userdata.hxx>
 
 #include "mat.hxx"
 
@@ -55,6 +58,10 @@ SG_USING_NAMESPACE(std);
 SG_USING_STD(string);
 
 
+// FIXME: should make this configurable
+static const bool sprite_lighting = true;
+
+
 // Constructor
 SGMaterialLib::SGMaterialLib ( void ) {
 }
@@ -99,6 +106,42 @@ static int gen_test_light_map() {
 #endif
 
 
+// generate a light sprite texture map
+static int gen_standard_light_sprite( int r, int g, int b, int alpha ) {
+    const int env_tex_res = 32;
+    int half_res = env_tex_res / 2;
+    unsigned char env_map[env_tex_res][env_tex_res][4];
+    GLuint tex_name;
+
+    for ( int i = 0; i < env_tex_res; ++i ) {
+        for ( int j = 0; j < env_tex_res; ++j ) {
+            double x = (i - half_res) / (double)half_res;
+            double y = (j - half_res) / (double)half_res;
+            double dist = sqrt(x*x + y*y);
+            if ( dist > 1.0 ) { dist = 1.0; }
+            double bright = cos( dist * SGD_PI_2 );
+            if ( bright < 0.01 ) { bright = 0.0; }
+            env_map[i][j][0] = r;
+            env_map[i][j][1] = g;
+            env_map[i][j][2] = b;
+            env_map[i][j][3] = (int)(bright * alpha);
+        }
+    }
+    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
+    glGenTextures( 1, &tex_name );
+    glBindTexture( GL_TEXTURE_2D, tex_name );
+  
+    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
+    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
+    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
+    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
+    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, env_tex_res, env_tex_res, 0,
+                  GL_RGBA, GL_UNSIGNED_BYTE, env_map);
+
+    return tex_name;
+}
+
+
 // generate standard colored directional light environment texture map
 static int gen_standard_dir_light_map( int r, int g, int b, int alpha ) {
     const int env_tex_res = 32;
@@ -188,17 +231,20 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
         throw;
     }
 
+    SGSharedPtr<SGMaterial> m;
+
     int nMaterials = materials.nChildren();
     for (int i = 0; i < nMaterials; i++) {
         const SGPropertyNode * node = materials.getChild(i);
         if (!strcmp(node->getName(), "material")) {
-            SGSharedPtr<SGMaterial> m = new SGMaterial( fg_root, node, season );
+            m = new SGMaterial( fg_root, node, season );
 
             vector<SGPropertyNode_ptr>names = node->getChildren("name");
             for ( unsigned int j = 0; j < names.size(); j++ ) {
                 string name = names[j]->getStringValue();
                 // cerr << "Material " << name << endl;
                 matlib[name] = m;
+                m->add_name(name);
                 SG_LOG( SG_TERRAIN, SG_INFO, "  Loading material "
                         << names[j]->getStringValue() );
             }
@@ -219,12 +265,18 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     gnd_lights->enable( GL_BLEND );
     gnd_lights->disable( GL_ALPHA_TEST );
     gnd_lights->disable( GL_LIGHTING );
-    matlib["GROUND_LIGHTS"] = new SGMaterial( gnd_lights );
+    m = new SGMaterial( gnd_lights );
+    m->add_name("GROUND_LIGHTS");
+    matlib["GROUND_LIGHTS"] = m;
 
     GLuint tex_name;
 
     // hard coded runway white light state
-    tex_name = gen_standard_dir_light_map( 235, 235, 195, 255 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 235, 195, 255 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 235, 195, 255 );
+    }
     ssgSimpleState *rwy_white_lights = new ssgSimpleState();
     rwy_white_lights->disable( GL_LIGHTING );
     rwy_white_lights->enable ( GL_CULL_FACE ) ;
@@ -237,14 +289,22 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_white_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_white_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_white_lights->setTexture( tex_name );
-    matlib["RWY_WHITE_LIGHTS"] = new SGMaterial( rwy_white_lights );
+    m = new SGMaterial( rwy_white_lights );
+    m->add_name("RWY_WHITE_LIGHTS");
+    matlib["RWY_WHITE_LIGHTS"] = m;
     // For backwards compatibility ... remove someday
-    matlib["RUNWAY_LIGHTS"] = new SGMaterial( rwy_white_lights );
-    matlib["RWY_LIGHTS"] = new SGMaterial( rwy_white_lights );
+    m->add_name("RUNWAY_LIGHTS");
+    matlib["RUNWAY_LIGHTS"] = m;
+    m->add_name("RWY_LIGHTS");
+    matlib["RWY_LIGHTS"] = m;
     // end of backwards compatitibilty
 
     // hard coded runway medium intensity white light state
-    tex_name = gen_standard_dir_light_map( 235, 235, 195, 205 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 235, 195, 205 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 235, 195, 205 );
+    }
     ssgSimpleState *rwy_white_medium_lights = new ssgSimpleState();
     rwy_white_medium_lights->disable( GL_LIGHTING );
     rwy_white_medium_lights->enable ( GL_CULL_FACE ) ;
@@ -257,11 +317,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_white_medium_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_white_medium_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_white_medium_lights->setTexture( tex_name );
-    matlib["RWY_WHITE_MEDIUM_LIGHTS"]
-        = new SGMaterial( rwy_white_medium_lights );
+    m = new SGMaterial( rwy_white_medium_lights );
+    m->add_name("RWY_WHITE_MEDIUM_LIGHTS");
+    matlib["RWY_WHITE_MEDIUM_LIGHTS"] = m;
 
     // hard coded runway low intensity white light state
-    tex_name = gen_standard_dir_light_map( 235, 235, 195, 155 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 235, 195, 155 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 235, 195, 155 );
+    }
     ssgSimpleState *rwy_white_low_lights = new ssgSimpleState();
     rwy_white_low_lights->disable( GL_LIGHTING );
     rwy_white_low_lights->enable ( GL_CULL_FACE ) ;
@@ -274,11 +339,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_white_low_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_white_low_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_white_low_lights->setTexture( tex_name );
-    matlib["RWY_WHITE_LOW_LIGHTS"]
-        = new SGMaterial( rwy_white_low_lights );
+    m = new SGMaterial( rwy_white_low_lights );
+    m->add_name("RWY_WHITE_LOW_LIGHTS");
+    matlib["RWY_WHITE_LOW_LIGHTS"] = m;
 
     // hard coded runway yellow light state
-    tex_name = gen_standard_dir_light_map( 235, 215, 20, 255 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 215, 20, 255 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 215, 20, 255 );
+    }
     ssgSimpleState *rwy_yellow_lights = new ssgSimpleState();
     rwy_yellow_lights->disable( GL_LIGHTING );
     rwy_yellow_lights->enable ( GL_CULL_FACE ) ;
@@ -291,10 +361,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_yellow_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_yellow_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_yellow_lights->setTexture( tex_name );
-    matlib["RWY_YELLOW_LIGHTS"] = new SGMaterial( rwy_yellow_lights );
+    m = new SGMaterial( rwy_yellow_lights );
+    m->add_name("RWY_YELLOW_LIGHTS");
+    matlib["RWY_YELLOW_LIGHTS"] = m;
 
     // hard coded runway medium intensity yellow light state
-    tex_name = gen_standard_dir_light_map( 235, 215, 20, 205 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 215, 20, 205 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 215, 20, 205 );
+    }
     ssgSimpleState *rwy_yellow_medium_lights = new ssgSimpleState();
     rwy_yellow_medium_lights->disable( GL_LIGHTING );
     rwy_yellow_medium_lights->enable ( GL_CULL_FACE ) ;
@@ -307,11 +383,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_yellow_medium_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_yellow_medium_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_yellow_medium_lights->setTexture( tex_name );
-    matlib["RWY_YELLOW_MEDIUM_LIGHTS"]
-        = new SGMaterial( rwy_yellow_medium_lights );
+    m = new SGMaterial( rwy_yellow_medium_lights );
+    m->add_name("RWY_YELLOW_MEDIUM_LIGHTS");
+    matlib["RWY_YELLOW_MEDIUM_LIGHTS"] = m;
 
     // hard coded runway low intensity yellow light state
-    tex_name = gen_standard_dir_light_map( 235, 215, 20, 155 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 215, 20, 155 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 215, 20, 155 );
+    }
     ssgSimpleState *rwy_yellow_low_lights = new ssgSimpleState();
     rwy_yellow_low_lights->disable( GL_LIGHTING );
     rwy_yellow_low_lights->enable ( GL_CULL_FACE ) ;
@@ -324,11 +405,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_yellow_low_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_yellow_low_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_yellow_low_lights->setTexture( tex_name );
-    matlib["RWY_YELLOW_LOW_LIGHTS"]
-        = new SGMaterial( rwy_yellow_low_lights );
+    m = new SGMaterial( rwy_yellow_low_lights );
+    m->add_name("RWY_YELLOW_LOW_LIGHTS");
+    matlib["RWY_YELLOW_LOW_LIGHTS"] = m;
 
     // hard coded runway red light state
-    tex_name = gen_standard_dir_light_map( 235, 90, 90, 255 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 90, 90, 255 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 90, 90, 255 );
+    }
     ssgSimpleState *rwy_red_lights = new ssgSimpleState();
     rwy_red_lights->disable( GL_LIGHTING );
     rwy_red_lights->enable ( GL_CULL_FACE ) ;
@@ -341,11 +427,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_red_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_red_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_red_lights->setTexture( tex_name );
-    matlib["RWY_RED_LIGHTS"]
-        = new SGMaterial( rwy_red_lights );
+    m = new SGMaterial( rwy_red_lights );
+    m->add_name("RWY_RED_LIGHTS");
+    matlib["RWY_RED_LIGHTS"] = m;
 
     // hard coded medium intensity runway red light state
-    tex_name = gen_standard_dir_light_map( 235, 90, 90, 205 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 90, 90, 205 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 90, 90, 205 );
+    }
     ssgSimpleState *rwy_red_medium_lights = new ssgSimpleState();
     rwy_red_medium_lights->disable( GL_LIGHTING );
     rwy_red_medium_lights->enable ( GL_CULL_FACE ) ;
@@ -358,11 +449,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_red_medium_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_red_medium_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_red_medium_lights->setTexture( tex_name );
-    matlib["RWY_RED_MEDIUM_LIGHTS"]
-        = new SGMaterial( rwy_red_medium_lights );
+    m = new SGMaterial( rwy_red_medium_lights );
+    m->add_name("RWY_RED_MEDIUM_LIGHTS");
+    matlib["RWY_RED_MEDIUM_LIGHTS"] = m;
 
     // hard coded low intensity runway red light state
-    tex_name = gen_standard_dir_light_map( 235, 90, 90, 155 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 90, 90, 155 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 90, 90, 155 );
+    }
     ssgSimpleState *rwy_red_low_lights = new ssgSimpleState();
     rwy_red_low_lights->disable( GL_LIGHTING );
     rwy_red_low_lights->enable ( GL_CULL_FACE ) ;
@@ -375,11 +471,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_red_low_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_red_low_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_red_low_lights->setTexture( tex_name );
-    matlib["RWY_RED_LOW_LIGHTS"]
-        = new SGMaterial( rwy_red_low_lights );
+    m = new SGMaterial( rwy_red_low_lights );
+    m->add_name("RWY_RED_LOW_LIGHTS");
+    matlib["RWY_RED_LOW_LIGHTS"] = m;
 
     // hard coded runway green light state
-    tex_name = gen_standard_dir_light_map( 20, 235, 20, 255 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 20, 235, 20, 255 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 20, 235, 20, 255 );
+    }
     ssgSimpleState *rwy_green_lights = new ssgSimpleState();
     rwy_green_lights->disable( GL_LIGHTING );
     rwy_green_lights->enable ( GL_CULL_FACE ) ;
@@ -392,11 +493,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_green_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_green_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_green_lights->setTexture( tex_name );
-    matlib["RWY_GREEN_LIGHTS"]
-        = new SGMaterial( rwy_green_lights );
+    m = new SGMaterial( rwy_green_lights );
+    m->add_name("RWY_GREEN_LIGHTS");
+    matlib["RWY_GREEN_LIGHTS"] = m;
 
     // hard coded medium intensity runway green light state
-    tex_name = gen_standard_dir_light_map( 20, 235, 20, 205 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 20, 235, 20, 205 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 20, 235, 20, 205 );
+    }
     ssgSimpleState *rwy_green_medium_lights = new ssgSimpleState();
     rwy_green_medium_lights->disable( GL_LIGHTING );
     rwy_green_medium_lights->enable ( GL_CULL_FACE ) ;
@@ -409,11 +515,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_green_medium_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_green_medium_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_green_medium_lights->setTexture( tex_name );
-    matlib["RWY_GREEN_MEDIUM_LIGHTS"]
-        = new SGMaterial( rwy_green_medium_lights );
+    m = new SGMaterial( rwy_green_medium_lights );
+    m->add_name("RWY_GREEN_MEDIUM_LIGHTS");
+    matlib["RWY_GREEN_MEDIUM_LIGHTS"] = m;
 
     // hard coded low intensity runway green light state
-    tex_name = gen_standard_dir_light_map( 20, 235, 20, 155 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 20, 235, 20, 155 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 20, 235, 20, 155 );
+    }
     ssgSimpleState *rwy_green_low_lights = new ssgSimpleState();
     rwy_green_low_lights->disable( GL_LIGHTING );
     rwy_green_low_lights->enable ( GL_CULL_FACE ) ;
@@ -426,13 +537,18 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_green_low_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     rwy_green_low_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     rwy_green_low_lights->setTexture( tex_name );
-    matlib["RWY_GREEN_LOW_LIGHTS"]
-        = new SGMaterial( rwy_green_low_lights );
-    matlib["RWY_GREEN_TAXIWAY_LIGHTS"]
-        = new SGMaterial( rwy_green_low_lights );
+    m = new SGMaterial( rwy_green_low_lights );
+    m->add_name("RWY_GREEN_LOW_LIGHTS");
+    matlib["RWY_GREEN_LOW_LIGHTS"] = m;
+    m->add_name("RWY_GREEN_TAXIWAY_LIGHTS");
+    matlib["RWY_GREEN_TAXIWAY_LIGHTS"] = m;
 
     // hard coded low intensity taxiway blue light state
-    tex_name = gen_taxiway_dir_light_map( 90, 90, 235, 205 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 90, 90, 235, 205 );
+    } else {
+        tex_name = gen_taxiway_dir_light_map( 90, 90, 235, 205 );
+    }
     ssgSimpleState *taxiway_blue_low_lights = new ssgSimpleState();
     taxiway_blue_low_lights->disable( GL_LIGHTING );
     taxiway_blue_low_lights->enable ( GL_CULL_FACE ) ;
@@ -445,11 +561,16 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     taxiway_blue_low_lights->setMaterial ( GL_SPECULAR, 0.0, 0.0, 0.0, 0.0 );
     taxiway_blue_low_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     taxiway_blue_low_lights->setTexture( tex_name );
-    matlib["RWY_BLUE_TAXIWAY_LIGHTS"]
-        = new SGMaterial( taxiway_blue_low_lights );
+    m = new SGMaterial( taxiway_blue_low_lights );
+    m->add_name("RWY_BLUE_TAXIWAY_LIGHTS");
+    matlib["RWY_BLUE_TAXIWAY_LIGHTS"] = m;
 
     // hard coded runway vasi light state
-    tex_name = gen_standard_dir_light_map( 235, 235, 195, 255 );
+    if ( sprite_lighting ) {
+        tex_name = gen_standard_light_sprite( 235, 235, 195, 255 );
+    } else {
+        tex_name = gen_standard_dir_light_map( 235, 235, 195, 255 );
+    }
     ssgSimpleState *rwy_vasi_lights = new ssgSimpleState();
     rwy_vasi_lights->disable( GL_LIGHTING );
     rwy_vasi_lights->enable ( GL_CULL_FACE ) ;
@@ -463,7 +584,9 @@ bool SGMaterialLib::load( const string &fg_root, const string& mpath, const char
     rwy_vasi_lights->setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0, 0.0 );
     // rwy_vasi_lights->setTexture( gen_vasi_light_map_old() );
     rwy_vasi_lights->setTexture( tex_name );
-    matlib["RWY_VASI_LIGHTS"] = new SGMaterial( rwy_vasi_lights );
+    m = new SGMaterial( rwy_vasi_lights );
+    m->add_name("RWY_VASI_LIGHTS");
+    matlib["RWY_VASI_LIGHTS"] = m;
 
     return true;
 }
@@ -491,6 +614,7 @@ bool SGMaterialLib::add_item ( const string &mat_name, const string &full_path )
 	    << mat_name << " (" << full_path << ")");
 
     matlib[mat_name] = new SGMaterial( full_path );
+    matlib[mat_name]->add_name(mat_name);
 
     return true;
 }
@@ -500,6 +624,7 @@ bool SGMaterialLib::add_item ( const string &mat_name, const string &full_path )
 bool SGMaterialLib::add_item ( const string &mat_name, ssgSimpleState *state )
 {
     matlib[mat_name] = new SGMaterial( state );
+    matlib[mat_name]->add_name(mat_name);
 
     SG_LOG( SG_TERRAIN, SG_INFO, "  Loading material given a premade "
 	    << "ssgSimpleState = " << mat_name );
@@ -538,3 +663,20 @@ void SGMaterialLib::load_next_deferred() {
 	  return;
     }
 }
+
+// Return the material from that given leaf
+const SGMaterial* SGMaterialLib::findMaterial(/*const*/ssgLeaf* leaf) const
+{
+  if (!leaf)
+    return 0;
+  
+  ssgBase* base = leaf->getUserData();
+  if (!base)
+    return 0;
+
+  SGMaterialUserData* matUserData = dynamic_cast<SGMaterialUserData*>(base);
+  if (!matUserData)
+    return 0;
+
+  return matUserData->getMaterial();
+}
