@@ -275,6 +275,7 @@ SGRangeAnimation::SGRangeAnimation (SGPropertyNode *prop_root,
 
 SGRangeAnimation::~SGRangeAnimation ()
 {
+   delete _condition;
 }
 
 int
@@ -359,7 +360,9 @@ SGSpinAnimation::SGSpinAnimation( SGPropertyNode *prop_root,
     _use_personality( props->getBoolValue("use-personality",false) ),
     _prop((SGPropertyNode *)prop_root->getNode(props->getStringValue("property", "/null"), true)),
     _last_time_sec( sim_time_sec ),
-    _condition(0)
+    _condition(0),
+    _factor( props, "factor", 1.0 ),
+    _position_deg( props, "starting-position-deg", 0.0 )
 {
     SGPropertyNode_ptr node = props->getChild("condition");
     if (node != 0)
@@ -394,42 +397,11 @@ SGSpinAnimation::SGSpinAnimation( SGPropertyNode *prop_root,
        _center[2] = props->getFloatValue("center/z-m", 0);
     }
     sgNormalizeVec3(_axis);
-
-    //_factor(props->getDoubleValue("factor", 1.0)),
-    _factor = 1.0;
-    _factor_min = 1.0;
-    _factor_max = 1.0;
-    SGPropertyNode_ptr factor_n = props->getNode( "factor" );
-    if ( factor_n != 0 ) {
-       SGPropertyNode_ptr rand_n = factor_n->getNode( "random" );
-       if ( rand_n != 0 ) {
-          _factor_min = rand_n->getDoubleValue( "min", 0.0 );
-          _factor_max = rand_n->getDoubleValue( "max", 1.0 );
-          _factor = _factor_min + sg_random() * ( _factor_max - _factor_min );
-       } else {
-          _factor = _factor_min = _factor_max = props->getDoubleValue("factor", 1.0);
-       }
-    }
-    //_position_deg(props->getDoubleValue("starting-position-deg", 0)),
-    _position_deg = 0.0;
-    _position_deg_min = 0.0;
-    _position_deg_max = 0.0;
-    SGPropertyNode_ptr position_deg_n = props->getNode( "starting-position-deg" );
-    if ( position_deg_n != 0 ) {
-       SGPropertyNode_ptr rand_n = position_deg_n->getNode( "random" );
-       if ( rand_n != 0 ) {
-          _position_deg_min = rand_n->getDoubleValue( "min", 0.0 );
-          _position_deg_max = rand_n->getDoubleValue( "max", 1.0 );
-          _position_deg = _position_deg_min + sg_random() * ( _position_deg_max - _position_deg_min );
-       } else {
-          _position_deg = _position_deg_min = _position_deg_max = 
-                  props->getDoubleValue("starting-position-deg", 1.0);
-       }
-    }
 }
 
 SGSpinAnimation::~SGSpinAnimation ()
 {
+   delete _condition;
 }
 
 int
@@ -438,17 +410,14 @@ SGSpinAnimation::update()
   if ( _condition == 0 || _condition->test() ) {
     double dt;
     float velocity_rpms;
-    if ( _use_personality ) {
+    if ( _use_personality && current_object ) {
       SGPersonalityBranch *key = current_object;
       if ( !key->getIntValue( this, INIT_SPIN ) ) {
-        double v = _factor_min + sg_random() * ( _factor_max - _factor_min );
-        key->setDoubleValue( v, this, FACTOR_SPIN );
+        key->setDoubleValue( _factor.shuffle(), this, FACTOR_SPIN );
+        key->setDoubleValue( _position_deg.shuffle(), this, POSITION_DEG_SPIN );
 
         key->setDoubleValue( sim_time_sec, this, LAST_TIME_SEC_SPIN );
         key->setIntValue( 1, this, INIT_SPIN );
-
-        v = _position_deg_min + sg_random() * ( _position_deg_max - _position_deg_min );
-        key->setDoubleValue( v, this, POSITION_DEG_SPIN );
       }
 
       _factor = key->getDoubleValue( this, FACTOR_SPIN );
@@ -545,7 +514,7 @@ SGTimedAnimation::init()
 int
 SGTimedAnimation::update()
 {
-    if ( _use_personality ) {
+    if ( _use_personality && current_object ) {
         SGPersonalityBranch *key = current_object;
         if ( !key->getIntValue( this, INIT_TIMED ) ) {
             double total = 0;
@@ -666,6 +635,7 @@ SGRotateAnimation::SGRotateAnimation( SGPropertyNode *prop_root,
 SGRotateAnimation::~SGRotateAnimation ()
 {
   delete _table;
+  delete _condition;
 }
 
 int
@@ -695,11 +665,12 @@ SGRotateAnimation::update()
 SGBlendAnimation::SGBlendAnimation( SGPropertyNode *prop_root,
                                         SGPropertyNode_ptr props )
   : SGAnimation(props, new ssgTransform),
+    _use_personality( props->getBoolValue("use-personality",false) ),
     _prop((SGPropertyNode *)prop_root->getNode(props->getStringValue("property", "/null"), true)),
     _table(read_interpolation_table(props)),
     _prev_value(1.0),
-    _offset(props->getDoubleValue("offset", 0.0)),
-    _factor(props->getDoubleValue("factor", 1.0)),
+    _offset(props,"offset",0.0),
+    _factor(props,"factor",1.0),
     _has_min(props->hasValue("min")),
     _min(props->getDoubleValue("min", 0.0)),
     _has_max(props->hasValue("max")),
@@ -717,6 +688,19 @@ SGBlendAnimation::update()
 {
   double _blend;
 
+  if ( _use_personality && current_object ) {
+    SGPersonalityBranch *key = current_object;
+    if ( !key->getIntValue( this, INIT_BLEND ) ) {
+      key->setDoubleValue( _factor.shuffle(), this, FACTOR_BLEND );
+      key->setDoubleValue( _offset.shuffle(), this, OFFSET_BLEND );
+
+      key->setIntValue( 1, this, INIT_BLEND );
+    }
+
+    _factor = key->getDoubleValue( this, FACTOR_BLEND );
+    _offset = key->getDoubleValue( this, OFFSET_BLEND );
+  }
+
   if (_table == 0) {
     _blend = 1.0 - (_prop->getDoubleValue() * _factor + _offset);
 
@@ -744,16 +728,17 @@ SGBlendAnimation::update()
 SGTranslateAnimation::SGTranslateAnimation( SGPropertyNode *prop_root,
                                         SGPropertyNode_ptr props )
   : SGAnimation(props, new ssgTransform),
-      _prop((SGPropertyNode *)prop_root->getNode(props->getStringValue("property", "/null"), true)),
-    _offset_m(props->getDoubleValue("offset-m", 0.0)),
-    _factor(props->getDoubleValue("factor", 1.0)),
+    _use_personality( props->getBoolValue("use-personality",false) ),
+    _prop((SGPropertyNode *)prop_root->getNode(props->getStringValue("property", "/null"), true)),
     _table(read_interpolation_table(props)),
     _has_min(props->hasValue("min-m")),
     _min_m(props->getDoubleValue("min-m")),
     _has_max(props->hasValue("max-m")),
     _max_m(props->getDoubleValue("max-m")),
     _position_m(props->getDoubleValue("starting-position-m", 0)),
-    _condition(0)
+    _condition(0),
+    _factor( props, "factor", 1.0 ),
+    _offset_m( props, "offset-m", 0.0 )
 {
   SGPropertyNode_ptr node = props->getChild("condition");
   if (node != 0)
@@ -768,12 +753,26 @@ SGTranslateAnimation::SGTranslateAnimation( SGPropertyNode *prop_root,
 SGTranslateAnimation::~SGTranslateAnimation ()
 {
   delete _table;
+  delete _condition;
 }
 
 int
 SGTranslateAnimation::update()
 {
   if (_condition == 0 || _condition->test()) {
+    if ( _use_personality && current_object ) {
+      SGPersonalityBranch *key = current_object;
+      if ( !key->getIntValue( this, INIT_TRANSLATE ) ) {
+        key->setDoubleValue( _factor.shuffle(), this, FACTOR_TRANSLATE );
+        key->setDoubleValue( _offset_m.shuffle(), this, OFFSET_TRANSLATE );
+      }
+
+      _factor = key->getDoubleValue( this, FACTOR_TRANSLATE );
+      _offset_m = key->getDoubleValue( this, OFFSET_TRANSLATE );
+
+      key->setIntValue( 1, this, INIT_TRANSLATE );
+    }
+
     if (_table == 0) {
       _position_m = (_prop->getDoubleValue() * _factor) + _offset_m;
       if (_has_min && _position_m < _min_m)
@@ -783,6 +782,7 @@ SGTranslateAnimation::update()
     } else {
       _position_m = _table->interpolate(_prop->getDoubleValue());
     }
+
     set_translation(_matrix, _position_m, _axis);
     ((ssgTransform *)_branch)->setTransform(_matrix);
   }
@@ -798,13 +798,14 @@ SGTranslateAnimation::update()
 SGScaleAnimation::SGScaleAnimation( SGPropertyNode *prop_root,
                                         SGPropertyNode_ptr props )
   : SGAnimation(props, new ssgTransform),
-      _prop((SGPropertyNode *)prop_root->getNode(props->getStringValue("property", "/null"), true)),
-    _x_factor(props->getDoubleValue("x-factor", 1.0)),
-    _y_factor(props->getDoubleValue("y-factor", 1.0)),
-    _z_factor(props->getDoubleValue("z-factor", 1.0)),
-    _x_offset(props->getDoubleValue("x-offset", 1.0)),
-    _y_offset(props->getDoubleValue("y-offset", 1.0)),
-    _z_offset(props->getDoubleValue("z-offset", 1.0)),
+    _use_personality( props->getBoolValue("use-personality",false) ),
+    _prop((SGPropertyNode *)prop_root->getNode(props->getStringValue("property", "/null"), true)),
+    _x_factor(props,"x-factor",1.0),
+    _y_factor(props,"y-factor",1.0),
+    _z_factor(props,"z-factor",1.0),
+    _x_offset(props,"x-offset",0.0),
+    _y_offset(props,"y-offset",0.0),
+    _z_offset(props,"z-offset",0.0),
     _table(read_interpolation_table(props)),
     _has_min_x(props->hasValue("x-min")),
     _has_min_y(props->hasValue("y-min")),
@@ -829,6 +830,27 @@ SGScaleAnimation::~SGScaleAnimation ()
 int
 SGScaleAnimation::update()
 {
+  if ( _use_personality && current_object ) {
+    SGPersonalityBranch *key = current_object;
+    if ( !key->getIntValue( this, INIT_SCALE ) ) {
+      key->setDoubleValue( _x_factor.shuffle(), this, X_FACTOR_SCALE );
+      key->setDoubleValue( _x_offset.shuffle(), this, X_OFFSET_SCALE );
+      key->setDoubleValue( _y_factor.shuffle(), this, Y_FACTOR_SCALE );
+      key->setDoubleValue( _y_offset.shuffle(), this, Y_OFFSET_SCALE );
+      key->setDoubleValue( _z_factor.shuffle(), this, Z_FACTOR_SCALE );
+      key->setDoubleValue( _z_offset.shuffle(), this, Z_OFFSET_SCALE );
+
+      key->setIntValue( 1, this, INIT_SCALE );
+    }
+
+    _x_factor = key->getDoubleValue( this, X_FACTOR_SCALE );
+    _x_offset = key->getDoubleValue( this, X_OFFSET_SCALE );
+    _y_factor = key->getDoubleValue( this, Y_FACTOR_SCALE );
+    _y_offset = key->getDoubleValue( this, Y_OFFSET_SCALE );
+    _z_factor = key->getDoubleValue( this, Z_FACTOR_SCALE );
+    _z_offset = key->getDoubleValue( this, Z_OFFSET_SCALE );
+  }
+
   if (_table == 0) {
       _x_scale = _prop->getDoubleValue() * _x_factor + _x_offset;
     if (_has_min_x && _x_scale < _min_x)
@@ -880,8 +902,13 @@ SGTexRotateAnimation::SGTexRotateAnimation( SGPropertyNode *prop_root,
       _min_deg(props->getDoubleValue("min-deg")),
       _has_max(props->hasValue("max-deg")),
       _max_deg(props->getDoubleValue("max-deg")),
-      _position_deg(props->getDoubleValue("starting-position-deg", 0))
+      _position_deg(props->getDoubleValue("starting-position-deg", 0)),
+      _condition(0)
 {
+  SGPropertyNode *node = props->getChild("condition");
+  if (node != 0)
+    _condition = sgReadCondition(prop_root, node);
+
   _center[0] = props->getFloatValue("center/x", 0);
   _center[1] = props->getFloatValue("center/y", 0);
   _center[2] = props->getFloatValue("center/z", 0);
@@ -894,11 +921,15 @@ SGTexRotateAnimation::SGTexRotateAnimation( SGPropertyNode *prop_root,
 SGTexRotateAnimation::~SGTexRotateAnimation ()
 {
   delete _table;
+  delete _condition;
 }
 
 int
 SGTexRotateAnimation::update()
 {
+  if (_condition && !_condition->test())
+    return 1;
+
   if (_table == 0) {
    _position_deg = _prop->getDoubleValue() * _factor + _offset_deg;
    if (_has_min && _position_deg < _min_deg)
@@ -931,8 +962,13 @@ SGTexTranslateAnimation::SGTexTranslateAnimation( SGPropertyNode *prop_root,
     _min(props->getDoubleValue("min")),
     _has_max(props->hasValue("max")),
     _max(props->getDoubleValue("max")),
-    _position(props->getDoubleValue("starting-position", 0))
+    _position(props->getDoubleValue("starting-position", 0)),
+    _condition(0)
 {
+  SGPropertyNode *node = props->getChild("condition");
+  if (node != 0)
+    _condition = sgReadCondition(prop_root, node);
+
   _axis[0] = props->getFloatValue("axis/x", 0);
   _axis[1] = props->getFloatValue("axis/y", 0);
   _axis[2] = props->getFloatValue("axis/z", 0);
@@ -942,11 +978,15 @@ SGTexTranslateAnimation::SGTexTranslateAnimation( SGPropertyNode *prop_root,
 SGTexTranslateAnimation::~SGTexTranslateAnimation ()
 {
   delete _table;
+  delete _condition;
 }
 
 int
 SGTexTranslateAnimation::update()
 {
+  if (_condition && !_condition->test())
+    return 1;
+
   if (_table == 0) {
     _position = (apply_mods(_prop->getDoubleValue(), _step, _scroll) + _offset) * _factor;
     if (_has_min && _position < _min)
@@ -1197,6 +1237,12 @@ SGMaterialAnimation::SGMaterialAnimation( SGPropertyNode *prop_root,
     _static_update = _update;
 }
 
+SGMaterialAnimation::~SGMaterialAnimation()
+{
+   delete _condition;
+}
+
+
 void SGMaterialAnimation::initColorGroup(SGPropertyNode_ptr group, ColorSpec *col, int flag)
 {
     if (!group)
@@ -1535,7 +1581,7 @@ SGShadowAnimation::SGShadowAnimation ( SGPropertyNode *prop_root,
 
 SGShadowAnimation::~SGShadowAnimation ()
 {
-	delete _condition;
+   delete _condition;
 }
 
 int
