@@ -1,14 +1,10 @@
 /*
-     $Id: custtrans.cxx,v 1.2 2004/05/11 22:39:21 curt Exp $
+     $Id: custtrans.cxx,v 1.4 2006-06-15 19:12:57 frohlich Exp $
 */
 
-#include "plib/ssg.h"
 #include "custtrans.hxx"
 void _ssgPushMatrix ( sgMat4 m );
 void _ssgPopMatrix  ();
-void _ssgReadInt     ( FILE *fd,                int   *var );
-void _ssgWriteInt    ( FILE *fd, const          int    var );
-extern sgMat4 _ssgOpenGLAxisSwapMatrix;
 
 void SGCustomTransform::copy_from( SGCustomTransform *src, int clone_flags )
 {
