@@ -17,9 +17,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: location.cxx,v 1.7 2005/09/05 13:23:55 ehofman Exp $
+// $Id: location.cxx,v 1.9 2006-03-08 18:16:09 mfranz Exp $
 
 
 #include <simgear/compiler.h>
@@ -102,7 +102,8 @@ static void MakeTRANS( sgMat4 dst, const double Theta,
 
 // Constructor
 SGLocation::SGLocation( void ):
-    _position_dirty(true), _orientation_dirty(true),
+    _orientation_dirty(true),
+    _position_dirty(true),
     _lon_deg(-1000),
     _lat_deg(0),
     _alt_ft(0),
