@@ -114,6 +114,9 @@ static int null_shader_callback( ssgEntity *e ) {
 }
 
 static int heat_haze_shader_callback( ssgEntity *e ) {
+   if( ! ((SGShadowAnimation *)e->getUserData())->get_condition_value() )
+       return true;
+
  	GLuint dlist = 0;
     ssgLeaf *leaf = (ssgLeaf *) e;
 #ifdef _SSG_USE_DLIST
@@ -312,6 +315,9 @@ static int heat_haze_shader_callback( ssgEntity *e ) {
 }
 
 static int fresnel_shader_callback( ssgEntity *e ) {
+   if( ! ((SGShadowAnimation *)e->getUserData())->get_condition_value() )
+       return true;
+
  	GLuint dlist = 0;
     ssgLeaf *leaf = (ssgLeaf *) e;
 #ifdef _SSG_USE_DLIST
@@ -385,6 +391,9 @@ static int fresnel_shader_callback( ssgEntity *e ) {
 
 
 static int chrome_shader_callback( ssgEntity *e ) {
+   if( ! ((SGShadowAnimation *)e->getUserData())->get_condition_value() )
+       return true;
+
  	GLuint dlist = 0;
     ssgLeaf *leaf = (ssgLeaf *) e;
 #ifdef _SSG_USE_DLIST
@@ -562,7 +571,9 @@ SGShaderAnimation::SGShaderAnimation ( SGPropertyNode *prop_root,
         glGetTexLevelParameteriv( GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &_texWidth);
         glGetTexLevelParameteriv( GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &_texHeight);
 
-        _textureData = new unsigned char[_texWidth * _texHeight * 3];
+        // W*H*3 should be enough, but it turned out that glGetTexImage() returns RGB
+        // and skips the A, so we need to calculate that in (nVidia driver 100.14.11)
+        _textureData = new unsigned char[_texWidth * _texHeight * 4];
         glGetTexImage(GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_BYTE, _textureData);
         glBindTexture(GL_TEXTURE_2D, 0 );
     }
@@ -609,7 +620,6 @@ void SGShaderAnimation::init()
 SGShaderAnimation::~SGShaderAnimation()
 {
     delete _condition;
-    delete _effectTexture;
     delete _textureData;
 }
 
