@@ -25,8 +25,6 @@
 #endif
 
 #include <plib/sg.h>
-#include <plib/ssg.h>
-#include <simgear/props/props.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/screen/extensions.hxx>
 #include <simgear/scene/model/animation.hxx>
@@ -647,8 +645,11 @@ static bool filterLeaf(ssgLeaf *this_kid) {
 	for( buff = lname; *leaf_name && l < (sizeof( lname )-1); ++buff, l++ )
 	     *buff = tolower(*leaf_name++);
 	*buff = 0;
-	if( !strncmp(lname, "noshadow", 8) )
+	if( !strncmp(lname, "noshadow", 8) ) {
+		SG_LOG(SG_ALL, SG_WARN, "use of deprecated noshadow prefix in '"
+				<< this_kid->getName() << '\'');
 		return false;
+	}
 	return true;
 }
 void SGShadowVolume::SceneryObject::traverseTree(ssgBranch *branch) {
@@ -936,6 +937,17 @@ void SGShadowVolume::addOccluder(ssgBranch *occluder, OccluderType occluder_type
 void SGShadowVolume::setupShadows( double lon, double lat,
 		double gst, double SunRightAscension, double SunDeclination, double sunAngle) {
 
+#ifdef __APPLE__
+	// FIXME: Find a better place for doing this.
+	// At the time SGShadowVolume is called, OpenGL extensions and 
+	// AlphaBits/StencilBits are not properly recognized on Macs. 
+	// so I simply re-initialize it when called for the first time
+	static bool re_initialized = false;
+	if (!re_initialized) {
+		this->init(sim_rendering);
+		re_initialized = true;
+	}
+#endif
 	shadowsAC_enabled = sim_rendering->getBoolValue("shadows-ac", false);
 	shadowsAC_transp_enabled = sim_rendering->getBoolValue("shadows-ac-transp", false);
 	shadowsAI_enabled = sim_rendering->getBoolValue("shadows-ai", false);
