@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: cloud.cxx,v 1.47 2006-03-26 08:22:26 mfranz Exp $
+// $Id: cloud.cxx,v 1.49.2.2 2007-12-12 17:38:19 timoore Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
@@ -36,7 +36,6 @@
 #include <simgear/math/polar3d.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/misc/sg_path.hxx>
 #include <simgear/screen/extensions.hxx>
 #include <simgear/screen/texture.hxx>
 #include <simgear/structure/ssgSharedPtr.hxx>
@@ -57,6 +56,9 @@
 #  endif
 #endif
 
+#if defined (__CYGWIN__)
+#include <ieeefp.h>
+#endif
 
 static ssgSharedPtr<ssgStateSelector> layer_states[SGCloudLayer::SG_MAX_CLOUD_COVERAGES];
 static bool state_initialized = false;
@@ -223,6 +225,7 @@ SGCloudLayer::SGCloudLayer( const string &tex_path ) :
     vl[0] = vl[1] = vl[2] = vl[3] = NULL;
     tl[0] = tl[1] = tl[2] = tl[3] = NULL;
     layer[0] = layer[1] = layer[2] = layer[3] = NULL;
+    sgSetVec2(base, sg_random(), sg_random());
 
     layer_root->addKid(layer_transform);
 	layer3D = new SGCloudField;
@@ -460,9 +463,6 @@ SGCloudLayer::rebuild()
             indices = new unsigned int[ 40 ];
         }
 
-        sgVec2 base;
-        sgSetVec2( base, sg_random(), sg_random() );
-
         const float layer_scale = layer_span / scale;
         const float layer_to_core = (SG_EARTH_RAD * 1000 + layer_asl);
         const float half_angle = 0.5 * layer_span / layer_to_core;
@@ -508,9 +508,6 @@ SGCloudLayer::rebuild()
         scale = 4000.0;
         last_lon = last_lat = -999.0f;
 
-        sgVec2 base;
-        sgSetVec2( base, sg_random(), sg_random() );
-
         // build the cloud layer
         sgVec4 color;
         sgVec3 vertex;
@@ -833,11 +830,11 @@ bool SGCloudLayer::reposition( sgVec3 p, sgVec3 up, double lon, double lat,
 }
 
 
-void SGCloudLayer::draw( bool top ) {
+void SGCloudLayer::draw( bool top, float *sun_color ) {
     if ( layer_coverage != SG_CLOUD_CLEAR ) {
 
 		if ( SGCloudField::enable3D && layer3D->is3D())
-			layer3D->Render();
+			layer3D->Render( sun_color );
 		else
         if ( bump_mapping && enable_bump_mapping ) {
 
