@@ -54,7 +54,10 @@ SG_USING_STD(vector);
 #  endif
 #endif
 
-extern SGSky *thesky;
+
+#if defined (__CYGWIN__)
+#include <ieeefp.h>
+#endif
 
 static list_of_culledCloud inViewClouds;
 
@@ -77,6 +80,8 @@ int SGCloudField::get_CacheResolution(void) {
 }
 
 void SGCloudField::set_CacheResolution(int resolutionPixels) {
+        if (resolutionPixels == 0)
+            resolutionPixels = 64;
 	if(cacheResolution == resolutionPixels)
 		return;
 	cacheResolution = resolutionPixels;
@@ -94,10 +99,10 @@ int SGCloudField::get_CacheSize(void) {
 
 void SGCloudField::set_CacheSize(int sizeKb) {
 	// apply in rendering option dialog
+        if (sizeKb == 0)
+            sizeKb = 1024;
 	if(last_cache_size == sizeKb)
 		return;
-	if(sizeKb == 0)
-		return;
 	if(sizeKb)
 		last_cache_size = sizeKb;
 	if(enable3D) {
@@ -360,7 +365,7 @@ void SGCloudField::cullClouds(sgVec3 eyePos, sgMat4 mat) {
 // because no field can have an infinite size (and we don't want to reach his border)
 // we draw this field and adjacent fields.
 // adjacent fields are not real, its the same field displaced by some offset
-void SGCloudField::Render(void) {
+void SGCloudField::Render( float *sun_color ) {
     sgVec3 eyePos;
 	double relx, rely;
 
@@ -400,7 +405,6 @@ void SGCloudField::Render(void) {
 	sgScaleVec3 ( SGNewCloud::ambLight, ambient , 1.1f);
 	// trying something else : clouds are more yellow/red at dawn/dusk
 	// and added a bit of blue ambient
-    float *sun_color = thesky->get_sun_color();
 	sgScaleVec3 ( SGNewCloud::sunlight, sun_color , 0.4f);
 	SGNewCloud::ambLight[2] += 0.1f;
 
