@@ -46,6 +46,7 @@
  * Nov. 2005, Use the simgear logging facility, Erik Hofman
  * Mar. 2006, Add MAC OS X support, Alexander Powell
  */
+#include <cstring>
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
@@ -146,8 +147,8 @@ RenderTexture::RenderTexture(const char *strMode)
     _pDisplay(NULL),
     _hGLContext(NULL),
     _hPBuffer(0),
-    _hPreviousContext(0),
     _hPreviousDrawable(0),
+    _hPreviousContext(0),
 #endif
     _iTextureTarget(GL_NONE),
     _iTextureID(0),
@@ -514,7 +515,7 @@ bool RenderTexture::Initialize(int width, int height,
         }
 
         if (error =
-           CGLChoosePixelFormat(&pixFormatAttribs[0], &pixFormat, &iNumFormats))
+           CGLChoosePixelFormat(&pixFormatAttribs[0], &pixFormat, (GLint *)&iNumFormats))
         {
             fprintf(stderr,
                     "RenderTexture Error: CGLChoosePixelFormat() failed.\n");
@@ -556,7 +557,7 @@ bool RenderTexture::Initialize(int width, int height,
        }
 
        long screen;
-       if (error = CGLGetVirtualScreen(CGLGetCurrentContext(), &screen))
+       if (error = CGLGetVirtualScreen(CGLGetCurrentContext(), (GLint *)&screen))
        {
            _cglCheckError(error);
            return false;
@@ -622,7 +623,7 @@ bool RenderTexture::Initialize(int width, int height,
     _pDisplay = glXGetCurrentDisplay();
     GLXContext context = glXGetCurrentContext();
     int screen = DefaultScreen(_pDisplay);
-    XVisualInfo *visInfo;
+    XVisualInfo *visInfo = NULL;
     
     GLXFBConfig *fbConfigs;
     int nConfigs;
@@ -667,9 +668,11 @@ bool RenderTexture::Initialize(int width, int height,
     }
     else
     {
+#ifdef WIN32
         int iFormat = 0;
         int iNumFormats;
         int attrib = 0;
+#endif
         for (int i=0;i<nConfigs;i++)
         {
             _hPBuffer = glXCreateGLXPbufferPtr(_pDisplay, fbConfigs[i], 
@@ -1119,7 +1122,7 @@ bool RenderTexture::EndCapture()
  */ 
 bool RenderTexture::BeginCapture(RenderTexture* current)
 {
-    bool bContextReset = false;
+    // bool bContextReset = false;
     
     if (current == this) {
         return true; // no switch necessary
@@ -1333,7 +1336,7 @@ void RenderTexture::_ParseModeString(const char *modeString,
         tokens.push_back(buf);
         buf = strtok(NULL, " ");
     }
-
+    free (mode);
     for (unsigned int i = 0; i < tokens.size(); i++)
     {
         string token = tokens[i];
@@ -2053,6 +2056,7 @@ void RenderTexture::_ParseModeString(const char *modeString,
                "Linux or MacOS");
 #endif 
     }
+//////////////////////////////////////////// END OF Function
 }
 
 //---------------------------------------------------------------------------
@@ -2623,8 +2627,8 @@ RenderTexture::RenderTexture(int width, int height,
     _iCurrentBoundBuffer(0),
     _iNumDepthBits(0),
     _iNumStencilBits(0),
-    _bDoubleBuffered(false),
     _bFloat(false),
+    _bDoubleBuffered(false),
     _bPowerOf2(true),
     _bRectangle(false),
     _bMipmap(false),
@@ -2644,8 +2648,8 @@ RenderTexture::RenderTexture(int width, int height,
     _pDisplay(NULL),
     _hGLContext(NULL),
     _hPBuffer(0),
-    _hPreviousContext(0),
     _hPreviousDrawable(0),
+    _hPreviousContext(0),
 #endif
     _iTextureTarget(GL_NONE),
     _iTextureID(0),
