@@ -16,7 +16,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: screen-dump.cxx,v 1.7 2006-03-08 18:16:09 mfranz Exp $
+// $Id: screen-dump.cxx,v 1.7.2.1 2007-06-14 20:20:59 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -71,8 +71,8 @@ bool sg_glWritePPMFile(const char *filename, GLubyte *buffer, int win_width, int
     fclose(fp);
     free(ibuffer);
 
-    printf("wrote file (%d x %d pixels, %d bytes)\n",
-	   win_width, win_height, RGB3*win_width*win_height);
+    printf("wrote file '%s' (%d x %d pixels, %d bytes)\n",
+	   filename, win_width, win_height, RGB3*win_width*win_height);
     return true;
 }
 
