@@ -18,6 +18,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include <cstring>
+#include <stdlib.h>
+
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
 #endif 
@@ -344,7 +347,7 @@ Shader::Shader(const char *name,const char *vertex,const char *fragment) {
 		
 		for(int i = 0; i < 8; i++) {
 			char texture[32];
-			sprintf(texture,"s_texture_%d",i);
+			snprintf(texture, 32, "s_texture_%d",i);
 			GLint location = glGetUniformLocationPtr(program,texture);
 			if(location >= 0) glUniform1iPtr(location,i);
 		}
@@ -483,7 +486,7 @@ void Shader::bind(const float *v,...) {
     }
 	const float *value = v;
 	va_list args;
-	va_start(args,value);
+	va_start(args,v);
 	for(int i = 0; i < (int)parameters.size(); i++) {
 		if( vertex_target ) {
 			glProgramLocalParameter4fvPtr( vertex_target, parameters[i].location, value);
@@ -549,7 +552,7 @@ void Shader::setParameter(const char *name,const float *value) {
 void Shader::setParameters(const float *v,...) {
 	const float *value = v;
 	va_list args;
-	va_start(args,value);
+	va_start(args,v);
 	for(int i = 0; i < (int)parameters.size(); i++) {
 		if( vertex_target ) {
 			glProgramLocalParameter4fvPtr( vertex_target, parameters[i].location, value);
