@@ -19,32 +19,30 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: xmlsound.cxx,v 1.17 2006-03-08 18:16:09 mfranz Exp $
+// $Id: xmlsound.cxx,v 1.19.2.3 2007-07-07 18:50:01 mfranz Exp $
+
+#ifdef HAVE_CONFIG_H
+#  include <simgear_config.h>
+#endif
 
 #include <simgear/compiler.h>
 
-#ifdef SG_HAVE_STD_INCLUDES
-#  include <cmath>
-#else
-#  include <math.h>
-#endif
 #include <string.h>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/props/condition.hxx>
-#include <simgear/math/fastmath.hxx>
+#include <simgear/math/SGMath.hxx>
 
 
 #include "xmlsound.hxx"
 
 
-#define LOG_ABS(x) (((x) > 1) ? (x) : (((x) < -1) ? -(x) : 0))
 // static double _snd_lin(double v)   { return v; }
 static double _snd_inv(double v)   { return (v == 0) ? 1e99 : 1/v; }
 static double _snd_abs(double v)   { return (v >= 0) ? v : -v; }
-static double _snd_sqrt(double v)  { return (v < 0) ? sqrt(-v) : sqrt(v); }
-static double _snd_log10(double v) { return fast_log10( LOG_ABS(v) ); }
-static double _snd_log(double v)   { return fast_log( LOG_ABS(v) ); }
+static double _snd_sqrt(double v)  { return sqrt(fabs(v)); }
+static double _snd_log10(double v) { return log10(fabs(v)); }
+static double _snd_log(double v)   { return log(fabs(v)); }
 // static double _snd_sqr(double v)   { return v*v; }
 // static double _snd_pow3(double v)  { return v*v*v; }
 
@@ -239,7 +237,7 @@ SGXmlSound::init(SGPropertyNode *root, SGPropertyNode *node, SGSoundMgr *sndmgr,
    SGPropertyNode_ptr pos = node->getChild("position");
    if ( pos != NULL ) {
        offset_pos[0] = pos->getDoubleValue("x", 0.0);
-       offset_pos[1] = pos->getDoubleValue("y", 0.0);
+       offset_pos[1] = -pos->getDoubleValue("y", 0.0);
        offset_pos[2] = pos->getDoubleValue("z", 0.0);
    }
 
@@ -254,7 +252,7 @@ SGXmlSound::init(SGPropertyNode *root, SGPropertyNode *node, SGSoundMgr *sndmgr,
    pos = node->getChild("orientation");
    if ( pos != NULL ) {
       dir[0] = pos->getDoubleValue("x", 0.0);
-      dir[1] = pos->getDoubleValue("y", 0.0);
+      dir[1] = -pos->getDoubleValue("y", 0.0);
       dir[2] = pos->getDoubleValue("z", 0.0);
       inner = pos->getDoubleValue("inner-angle", 360.0);
       outer = pos->getDoubleValue("outer-angle", 360.0);
@@ -274,7 +272,8 @@ SGXmlSound::init(SGPropertyNode *root, SGPropertyNode *node, SGSoundMgr *sndmgr,
        // "alSource".  The semantics of what is going on here seems
        // confused and needs to be thought through more carefully.
         _sample = new SGSoundSample( path.c_str(),
-                                    node->getStringValue("path", "") );
+                                    node->getStringValue("path", ""),
+                                    false );
 
        _mgr->add( _sample, _name );
    }
@@ -426,15 +425,15 @@ SGXmlSound::update (double dt)
    //
    // Change sample state
    //
+
+   double vol = volume_offset + volume;
+   if (vol > 1.0) {
+      SG_LOG(SG_GENERAL, SG_WARN, "Sound volume too large for '"
+              << _name << "':  " << vol << "  ->  clipping to 1.0");
+      vol = 1.0;
+   }
+   _sample->set_volume(vol);
    _sample->set_pitch( pitch_offset + pitch );
-   if ((volume_offset + volume ) > 1.0)
-   {
-      _sample->set_volume( 1.0 );
-      SG_LOG(SG_GENERAL, SG_WARN,
-             "Volume larger than 1.0 in configuration for '" << _name
-              << "', clipping.");
-   } else 
-      _sample->set_volume( volume_offset + volume );
 
 
    //
